@@ -22,7 +22,7 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
 */
 
 /* MAKING CHANGES TO THIS FILE??   READ THIS FIRST!!!
@@ -57,7 +57,6 @@
 #include <sys/types.h>
 #include <sys/mman.h>
 #include "../libv4lconvert/libv4lsyscall-priv.h"
-#include <linux/videodev.h>
 #include <linux/videodev2.h>
 #include <libv4l2.h>
 #include "libv4l1.h"
@@ -87,6 +86,10 @@ static unsigned int palette_to_pixelformat(unsigned int palette)
 	case VIDEO_PALETTE_RGB565:
 		return V4L2_PIX_FMT_RGB565;
 	case VIDEO_PALETTE_RGB24:
+	        /* Yes v4l1 RGB24 really is BGR24, this is a weirdness in the
+	           V4L1 API. Which unfortunately some V4L1 apps get wrong.
+	           If you have an app which has r and b swapped and end up here
+	           fix the app, this is not a libv4l1 bug */
 		return V4L2_PIX_FMT_BGR24;
 	case VIDEO_PALETTE_RGB32:
 		return V4L2_PIX_FMT_BGR32;
@@ -119,6 +122,7 @@ static unsigned int pixelformat_to_palette(unsigned int pixelformat)
 	case V4L2_PIX_FMT_RGB565:
 		return VIDEO_PALETTE_RGB565;
 	case V4L2_PIX_FMT_BGR24:
+	        /* See the comment in palette_to_pixelformat above */
 		return VIDEO_PALETTE_RGB24;
 	case V4L2_PIX_FMT_BGR32:
 		return VIDEO_PALETTE_RGB32;
@@ -317,11 +321,6 @@ int v4l1_open(const char *file, int oflag, ...)
 	if (SYS_IOCTL(fd, VIDIOC_QUERYCAP, &cap2))
 		return fd;
 
-	/* IMPROVEME */
-	/* we only support simple video capture devices which do not do overlay */
-	if ((cap2.capabilities & 0x0F) != V4L2_CAP_VIDEO_CAPTURE)
-		return fd;
-
 	/* If no log file was set by the app, see if one was specified through the
 	   environment */
 	if (!v4l1_log_file) {
@@ -395,7 +394,7 @@ int v4l1_open(const char *file, int oflag, ...)
 		devices[index].flags |= V4L1_SUPPORTS_ENUMINPUT;
 
 	standard2.index = 0;
-	if (v4l2_ioctl(fd, VIDIOC_ENUMSTD, &input2) == 0)
+	if (v4l2_ioctl(fd, VIDIOC_ENUMSTD, &standard2) == 0)
 		devices[index].flags |= V4L1_SUPPORTS_ENUMSTD;
 
 	V4L1_LOG("open: %d\n", fd);
